@@ -565,6 +565,11 @@ proc HttpdRead {sock} {
 		    }
 		}
 
+		# Flag the need to check for an extra newline
+		# in SSL connections by some browsers.
+
+		set data(checkNewline) 1
+
 		# Facilitate a backdoor hook between Url_DecodeQuery
 		# where it will read the post data on behalf of the
 		# domain handler in the case where the domain handler
@@ -637,21 +642,14 @@ proc Httpd_PostDataSize {sock} {
 #	The amount of data left to read.  When this goes to zero, you are done.
 
 proc Httpd_GetPostData {sock varName {size -1}} {
+    global Httpd
     upvar #0 Httpd$sock data
     upvar 1 $varName buffer
 
-    if {$size < 0 || $size > $data(count)} {
-	set size $data(count)
-    }
-    set block [read $sock $size]
-    append buffer $block
-    set data(count) [expr {$data(count) - [string length $block]}]
-    if {[eof $sock]} {
-	set data(count) 0
-    }
-    if {$data(count) == 0} {
-	Url_PostHook $sock 0
+    if {$size < 0} {
+	set size $Httpd(bufsize)
     }
+    HttpdReadPost $sock buffer $size
     return $data(count)
 }
 
@@ -666,6 +664,7 @@ proc Httpd_GetPostData {sock varName {size -1}} {
 #		must be a global or fully scoped namespace variable, or
 #		this can be the empty string, in which case the data
 #		is discarded.
+#	blockSize	Default read block size.
 #	cmd	Callback to make when the post data has been read.
 #		It is called like this:
 #		cmd $sock $varName $errorString
@@ -676,19 +675,31 @@ proc Httpd_GetPostData {sock varName {size -1}} {
 #	asynchronously.  The data is appened to the named variable.
 #	The callback is made 
 
-proc Httpd_GetPostDataAsync {sock varName cmd} {
-    fileevent $sock readable [list HttpdReadPost $sock $varName $cmd]
+proc Httpd_GetPostDataAsync {sock varName blockSize cmd} {
+    fileevent $sock readable \
+	[list HttpdReadPostGlobal $sock $varName $blockSize $cmd]
     return
 }
 
-proc HttpdReadPost {sock varName cmd} {
+proc HttpdReadPostGlobal {sock varName blockSize {cmd {}}} {
+
+    # This fileevent callback can only access a global variable.
+    # But HttpdReadPost needs to affect a local variable in its
+    # caller so it can be shared with Httpd_GetPostData.
+    # So, the fileevent case has an extra procedure frame.
+
+    upvar #0 $varName buffer
+    HttpdReadPost $sock buffer $blockSize $cmd
+}
+
+proc HttpdReadPost {sock varName blockSize {cmd {}}} {
     global Httpd
     upvar #0 Httpd$sock data
 
     # Ensure that the variable, if specified, exists by appending "" to it
 
     if {[string length $varName]} {
-	upvar #0 $varName buffer
+	upvar 1 $varName buffer
 	append buffer ""
     }
 
@@ -700,8 +711,27 @@ proc HttpdReadPost {sock varName cmd} {
 	    set doneMsg ""
 	}
     } else {
-	set toRead [expr {$data(count) > $Httpd(bufsize) ? \
-		$Httpd(bufsize) : $data(count)}]
+	if {[info exist data(checkNewline)]} {
+
+	    # Gobble a single leading \n from the POST data
+	    # This is generated by various versions of Netscape
+	    # when using https/SSL.  This extra \n is not counted
+	    # in the content-length (thanks!)
+
+	    set nl [read $sock 1]
+	    if {[string compare $nl \n] != 0} {
+
+		# It was not an extra newline.
+
+		incr data(count) -1
+		if {[info exist buffer]} {
+		    append buffer $nl
+		}
+	    }
+	    unset data(checkNewline)
+	}
+	set toRead [expr {$data(count) > $blockSize ? \
+		$blockSize : $data(count)}]
 	if {[catch {read $sock $toRead} block]} {
 	    set doneMsg $block
 	} else {
@@ -716,8 +746,13 @@ proc HttpdReadPost {sock varName cmd} {
     }
     if {[info exist doneMsg]} {
 	Url_PostHook $sock 0
-	eval $cmd [list $sock $varName $doneMsg]
+	if {[string length $cmd]} {
+	    eval $cmd [list $sock $varName $doneMsg]
+	}
 	catch {fileevent $sock readable {}}
+	return $doneMsg
+    } else {
+	return ""
     }
 }
 
